import {RequestDecorator} from '../../../common';
import {DataSource} from '../../../data-source';
import {Resource} from '../../../../common';
import {BackendResource} from '../../../core';

declare module '../../../common' {
	interface RequestContext {
		resource?: Resource;
		resourceId?: string;
	}
}

export const decorateRequestWithResource: RequestDecorator = (req) => {
	const [, resourceRouteName, resourceId = ''] = req.url?.split('/') ?? [];
	const resource = Array.from(req.backend.app.resources)
		.find((r) => r.state.routeName === resourceRouteName) as BackendResource | undefined;

	if (typeof resource !== 'undefined') {
		req.resource = resource;
		req.resource.dataSource = req.backend.dataSource(req.resource) as DataSource;
		req.resourceId = resourceId;
	}

	return req;
};
